/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authorization.manager;


import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.sf.jguard.core.authorization.AuthorizationException;





/**
 * permission manager factory, which returns the AuthorizationManager implementation.
 * this class is loaded by the jvm classloader. we cannot used commons-logging library
 * loaded by the application sevrer classloader.
 * @author <a href="mailto:diabolo512@users.sourceforge.net ">Charles Gay</a>
 *
 */
public class AuthorizationManagerFactory {
	/** Logger for this class */
	private static final Logger logger = Logger.getLogger(AuthorizationManagerFactory.class.getName());
        private static AuthorizationManager authorizationManager = null;


    /**
     * define the AuthorizationManager implementation and return it.
     * @param authorizationManagerOptions
     * @return <strong>true</strong> if creation succeed,
     * <strong>false</strong> if one
     * @throws AuthorizationException
     */
    public static AuthorizationManager createAuthorizationManager(Class authorizationManagerImpl,Map authorizationManagerOptions) throws AuthorizationException{
                
		try {
                       Constructor constructor  = authorizationManagerImpl.getConstructor(new Class[]{Map.class});
                       authorizationManager = (AuthorizationManager)constructor.newInstance(new Object[]{authorizationManagerOptions});
                       logger.log(Level.FINEST,"authorizationManagerOptions="+authorizationManagerOptions);
                } catch (InstantiationException e1) {
			if (logger.isLoggable(Level.FINEST)) {
				logger.log(Level.FINEST,"createAuthorizationManager() -  AuthorizationManager cannot be instantiated : InstantiationException");
			}
                        throw new AuthorizationException(" AuthorizationManager cannot be created : InstantiationException",e1);
		} catch (IllegalAccessException e1) {
			if (logger.isLoggable(Level.FINEST)) {
				logger.log(Level.FINEST,"createAuthorizationManager() - AuthorizationManager cannot be accessed : IllegalAccessException");
			}
                        throw new AuthorizationException(" AuthorizationManager cannot be accessed: IllegalAccessException",e1);
		}catch(SecurityException sex){
                   	if (logger.isLoggable(Level.FINEST)) {
        			logger.log(Level.FINEST,"createAuthorizationManager() -  AuthorizationManager cannot be accessed : SecurityException");
			}
                    throw new AuthorizationException(" AuthorizationManager cannot be accessed: SecurityException",sex);
                }catch(NoSuchMethodException nse){
                    if (logger.isLoggable(Level.FINEST)) {
			logger.log(Level.FINEST,"createAuthorizationManager() -  constructor with map argument does not exists: nosuchmethodexception");
		    }
                    throw new AuthorizationException(" AuthorizationManager cannot be accessed: SecurityException",nse);
                }catch(InvocationTargetException ite){
                    if (logger.isLoggable(Level.FINEST)) {
			logger.log(Level.FINEST,"createAuthorizationManager() - createAuthorizationManager cannot be invoked : InvocationTargetException");
		    }
                    throw new AuthorizationException(" AuthorizationManager cannot be invoked: InvocationTargetException",ite);
                }		
		return authorizationManager;

    }


    /**
     * @return authorizationManager or <code><strong>null</strong></code> when no authorizationManager is already created
     */
	public static AuthorizationManager getAuthorizationManager() {
		return authorizationManager;
	}
        
        public static void setAuthorizationManager(
			AuthorizationManager authorizationManager) {
		AuthorizationManagerFactory.authorizationManager = authorizationManager;
	}

}
