/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authorization.permissions;

import java.net.URL;
import java.security.CodeSource;
import java.security.Permission;
import java.security.PermissionCollection;
import java.security.ProtectionDomain;
import java.security.cert.Certificate;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.security.auth.Subject;
import net.sf.jguard.core.authorization.policy.ProtectionDomainUtils;

/**
* Audit permissions checks.
* @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
* @since 1.1*/
public class AuditPermissionCollection extends PermissionCollection{

    private PermissionCollection pm;
    private ProtectionDomain pDomain;
    private CodeSource cs;
    private static Logger logger = Logger.getLogger(AuditPermissionCollection.class.getName());
    private boolean protectionDomainMode;
    
    /**
     * constructor used to track permissions check against user identified by a {@link ProtectionDomain}.
     * @param permissionCollection
     * @param protectionDomain
     */
    public AuditPermissionCollection(PermissionCollection permissionCollection,ProtectionDomain protectionDomain){
        this.pm = permissionCollection;
        this.pDomain = protectionDomain;
        protectionDomainMode = true;
    }
    
    /**
     * constructor used to track permissions check against jars identified by a {@link CodeSource}.
     * @param permissionCollection
     * @param codeSource
     */
    public AuditPermissionCollection(PermissionCollection permissionCollection,CodeSource codeSource){
        this.pm = permissionCollection;
        this.cs = codeSource;
        protectionDomainMode = false;
    }
    public void add(Permission permission) {
        pm.add(permission);
    }

    public boolean implies(Permission permission) {
        boolean result = pm.implies(permission);
        if(protectionDomainMode){
            //we audit user permissions check
            //logger.log(SecurityLevel.JGUARD_SECURITY, "permission check result={1}", new Object[]{Boolean.valueOf(result).toString()});
            Subject subject = ProtectionDomainUtils.getSubject(pDomain);
            logPermissionCollection(pm);
            logger.log(Level.FINEST," ProtectionDomain permission check "+permission.toString()+" result="+Boolean.valueOf(result));
        }else if (cs!=null){
            //we audit code permissions check
            Certificate[] certs = cs.getCertificates();
            URL locationl = cs.getLocation();
            //CodeSigner[] codeSigners = cs.getCodeSigners(); //only in JDK 5 and igher
            logger.log(Level.FINEST," CodeSource permission check "+permission.toString()+" result="+Boolean.valueOf(result));
        }
            return result;
        
    }

    public Enumeration elements() {
        return pm.elements();
    }
    
    private void logPermissionCollection(PermissionCollection pm){
        Enumeration enumPerm = pm.elements();
        while(enumPerm.hasMoreElements()){
            Permission perm =(Permission)enumPerm.nextElement();
            logger.log(Level.FINEST,"@@ user has got "+perm.getClass().getName()+" name="+perm.getName()+" actions="+perm.getActions()+" @@");
        }
    }

}
